@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """Magic functions for InteractiveShell.
 
-$Id: Magic.py,v 1.53 2004/12/17 23:57:18 fperez Exp $"""
+$Id: Magic.py,v 1.64 2005/04/12 17:35:23 fperez Exp $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001 Janko Hauser <jhauser@zscout.de> and
@@ -21,7 +21,11 @@ __license__ = Release.license
 
 # Python standard modules
 import __builtin__
-import os,sys,inspect,pydoc,re,tempfile,profile,pstats,shlex,pdb,bdb
+import os,sys,inspect,pydoc,re,tempfile,shlex,pdb,bdb,time
+try:
+    import profile,pstats
+except ImportError:
+    profile = pstats = None
 from getopt import getopt
 from pprint import pprint, pformat
 from cStringIO import StringIO
@@ -30,8 +34,8 @@ from cStringIO import StringIO
 from IPython.Struct import Struct
 from IPython.Itpl import Itpl, itpl, printpl,itplns
 from IPython.FakeModule import FakeModule
-from IPython.genutils import *
 from IPython import OInspect
+from IPython.genutils import *
 
 # Globals to be set later by Magic constructor
 MAGIC_PREFIX = ''
@@ -161,7 +165,15 @@ class Magic:
         self.options_table = {}
         MAGIC_PREFIX = shell.name+'.magic_'
         MAGIC_ESCAPE = shell.ESC_MAGIC
+        if profile is None:
+            self.magic_prun = self.profile_missing_notice
 
+    def profile_missing_notice(self, *args, **kwargs):
+        error("""\
+The profile module could not be found.  If you are a Debian user,
+it has been removed from the standard Debian package because of its non-free
+license. To use profiling, please install"python2.3-profiler" from non-free.""")
+    
     def default_option(self,fn,optstr):
         """Make an entry in the options_table for fn, with value optstr"""
         
@@ -219,7 +231,7 @@ class Magic:
     def _ofind(self,oname):
         """Find an object in the available namespaces.
 
-        self._ofind(oname) -> dict with keys: found,obj,ospace,docstring,ismagic
+        self._ofind(oname) -> dict with keys: found,obj,ospace,ismagic
 
         Has special code to detect magic functions.
         """
@@ -287,15 +299,8 @@ class Magic:
             found = 1
             ospace = 'Interactive'
             
-        # Get the object's docstring
-        if found:
-            if isalias:
-                ds = "Alias to the system command:\n  %s" % obj[1]
-            else:
-                ds = OInspect.getdoc(obj)
-
         return {'found':found, 'obj':obj, 'namespace':ospace,
-                'docstring':ds, 'ismagic':ismagic, 'isalias':isalias}
+                'ismagic':ismagic, 'isalias':isalias}
         
     def arg_err(self,func):
         """Print docstring if incorrect arguments were passed"""
@@ -317,7 +322,7 @@ class Magic:
         # Paragraph continue
         par_re = re.compile(r'\\$',re.MULTILINE)
 
-        str = cmd_name_re.sub(r'\n\\texttt{\\textbf{\1}}:',str)
+        str = cmd_name_re.sub(r'\n\\texttt{\\textsl{\\large \1}}:',str)
         str = cmd_re.sub(r'\\texttt{\g<cmd>}',str)
         str = par_re.sub(r'\\\\',str)
         str = escape_re.sub(r'\\\1',str)
@@ -514,12 +519,12 @@ Currently the magic system has the following functions:\n"""
         print "System verbose printing is:",\
               ['OFF','ON'][self.shell.rc.system_verbose]
 
-    def magic_hist(self, parameter_s = ''):
+    def magic_history(self, parameter_s = ''):
         """Print input history (_i<n> variables), with most recent last.
         
-        %hist [-n]       -> print at most 40 inputs (some may be multi-line)\\
-        %hist [-n] n     -> print at most n inputs\\
-        %hist [-n] n1 n2 -> print inputs between n1 and n2 (n2 not included)\\
+        %history [-n]       -> print at most 40 inputs (some may be multi-line)\\
+        %history [-n] n     -> print at most n inputs\\
+        %history [-n] n1 n2 -> print inputs between n1 and n2 (n2 not included)\\
 
         Each input's number <n> is shown, and is accessible as the
         automatically generated variable _i<n>.  Multi-line statements are
@@ -564,6 +569,10 @@ Currently the magic system has the following functions:\n"""
             else:
                 print inline,
 
+    def magic_hist(self, parameter_s=''):
+        """Alternate name for %history."""
+        return self.magic_history(parameter_s)
+
     def magic_p(self, parameter_s=''):
         """Just a short alias for Python's 'print'."""
         exec 'print ' + parameter_s in self.shell.user_ns
@@ -589,11 +598,9 @@ Currently the magic system has the following functions:\n"""
         # Look through the input history in reverse
         for n in range(len(self.shell.input_hist)-2,0,-1):
             input = self.shell.input_hist[n]
-            if input.startswith('#'+esc_magic):
-                input = input[1:]
             # skip plain 'r' lines so we don't recurse to infinity
-            if input != '%sr\n' % esc_magic and \
-               (input.startswith(start) or input.startswith(start_magic)):
+            if input != 'ipmagic("r")\n' and \
+                   (input.startswith(start) or input.startswith(start_magic)):
                 #print 'match',`input`  # dbg
                 if input.startswith(esc_magic):
                     input = magic2python(input)
@@ -687,11 +694,16 @@ Currently the magic system has the following functions:\n"""
         """Provide detailed information about an object.
 
         '%pinfo object' is just a synonym for object? or ?object."""
-        
+
+        #print 'pinfo par: <%s>' % parameter_s  # dbg
+
         # detail_level: 0 -> obj? , 1 -> obj??
         detail_level = 0
-        qmark1,oname,qmark2 = re.match('(\?*)(.*?)(\??$)',parameter_s).groups()
-        if qmark1 or qmark2:
+        # We need to detect if we got called as 'pinfo pinfo foo', which can
+        # happen if the user types 'pinfo foo?' at the cmd line.
+        pinfo,qmark1,oname,qmark2 = \
+               re.match('(pinfo )?(\?*)(.*?)(\??$)',parameter_s).groups()
+        if pinfo or qmark1 or qmark2:
             detail_level = 1
         self._inspect('pinfo',oname,detail_level=detail_level)
 
@@ -975,6 +987,7 @@ Currently the magic system has the following functions:\n"""
         print 'Automatic pdb calling has been turned',\
               on_off(self.shell.InteractiveTB.call_pdb)
 
+
     def magic_prun(self, parameter_s ='',user_mode=1,
                    opts=None,arg_lst=None,prog_ns=None):
 
@@ -1080,6 +1093,9 @@ Currently the magic system has the following functions:\n"""
           In [1]: import profile; profile.help() """
 
         opts_def = Struct(D=[''],l=[],s=['time'],T=[''])
+        # protect user quote marks
+        parameter_s = parameter_s.replace('"',r'\"').replace("'",r"\'")
+        
         if user_mode:  # regular user call
             opts,arg_str = self.parse_options(parameter_s,'D:l:rs:T:',
                                               list_all=1)
@@ -1374,7 +1390,73 @@ Currently the magic system has the following functions:\n"""
         force any file to be treated as a log file."""
 
         for f in parameter_s.split():
-            self.shell.safe_execfile(f,self.shell.user_ns,self.shell.user_ns,islog=1)
+            self.shell.safe_execfile(f,self.shell.user_ns,
+                                     self.shell.user_ns,islog=1)
+
+    def magic_time(self,parameter_s = ''):
+        """Time execution of a Python statement or expression.
+
+        The CPU and wall clock times are printed, and the value of the
+        expression (if any) is returned.  Note that under Win32, system time
+        is always reported as 0, since it can not be measured.
+
+        This function provides very basic timing functionality.  In Python
+        2.3, the timeit module offers more control and sophistication, but for
+        now IPython supports Python 2.2, so we can not rely on timeit being
+        present.
+        
+        Some examples:
+
+          In [1]: time 2**128
+          CPU times: user 0.00 s, sys: 0.00 s, total: 0.00 s
+          Wall time: 0.00
+          Out[1]: 340282366920938463463374607431768211456L
+
+          In [2]: n = 1000000
+
+          In [3]: time sum(range(n))
+          CPU times: user 1.20 s, sys: 0.05 s, total: 1.25 s
+          Wall time: 1.37
+          Out[3]: 499999500000L
+
+          In [4]: time print 'hello world'
+          hello world
+          CPU times: user 0.00 s, sys: 0.00 s, total: 0.00 s
+          Wall time: 0.00
+          """
+        
+        # fail immediately if the given expression can't be compiled
+        try:
+            mode = 'eval'
+            code = compile(parameter_s,'<timed eval>',mode)
+        except SyntaxError:
+            mode = 'exec'
+            code = compile(parameter_s,'<timed exec>',mode)
+        # skew measurement as little as possible
+        glob = self.shell.user_ns
+        clk = clock2
+        wtime = time.time
+        # time execution
+        wall_st = wtime()
+        if mode=='eval':
+            st = clk()
+            out = eval(code,glob)
+            end = clk()
+        else:
+            st = clk()
+            exec code in glob
+            end = clk()
+            out = None
+        wall_end = wtime()
+        # Compute actual times and report
+        wall_time = wall_end-wall_st
+        cpu_user = end[0]-st[0]
+        cpu_sys = end[1]-st[1]
+        cpu_tot = cpu_user+cpu_sys
+        print "CPU times: user %.2f s, sys: %.2f s, total: %.2f s" % \
+              (cpu_user,cpu_sys,cpu_tot)
+        print "Wall time: %.2f" % wall_time
+        return out
 
     def magic_macro(self,parameter_s = ''):
         """Define a set of input lines as a macro for future re-execution.
@@ -1476,11 +1558,13 @@ Currently the magic system has the following functions:\n"""
         Usage:
           %edit [options] [args]
 
-        %edit will use the editor you have configured in your environment as
-        the EDITOR variable. If this isn't found, it will default to vi under
-        Linux/Unix and to notepad under Windows.
+        %edit runs IPython's editor hook.  The default version of this hook is
+        set to call the __IPYTHON__.rc.editor command.  This is read from your
+        environment variable $EDITOR.  If this isn't found, it will default to
+        vi under Linux/Unix and to notepad under Windows.  See the end of this
+        docstring for how to change the editor hook.
 
-        You can also set the value of this editor via the commmand-line option
+        You can also set the value of this editor via the command line option
         '-editor' or in your ipythonrc file. This is useful if you wish to use
         specifically for IPython an editor different from your typical default
         (and for Windows users who typically don't set environment variables).
@@ -1522,9 +1606,9 @@ Currently the magic system has the following functions:\n"""
         edit it and have the file be executed automatically.
 
         Note: opening at an exact line is only supported under Unix, and some
-        editors (like kedit and gedit) do not understand the '+NUMBER'
-        parameter necessary for this feature. Good editors like (X)Emacs, vi,
-        jed, pico and joe all do.
+        editors (like kedit and gedit up to Gnome 2.8) do not understand the
+        '+NUMBER' parameter necessary for this feature. Good editors like
+        (X)Emacs, vi, jed, pico and joe all do.
 
         - If the argument is not found as a variable, IPython will look for a
         file with that name (adding .py if necessary) and load it into the
@@ -1544,7 +1628,7 @@ Currently the magic system has the following functions:\n"""
 
         In [1]: ed\\
         Editing... done. Executing edited code...\\
-        Out[1]: 'def foo():\n    print "foo() was defined in an editing session"\n'
+        Out[1]: 'def foo():\\n    print "foo() was defined in an editing session"\\n'
 
         We can then call the function foo():
         
@@ -1583,7 +1667,17 @@ Currently the magic system has the following functions:\n"""
         Editing... done. Executing edited code...\\
         hello again\\
         Out[10]: "print 'hello again'\\n"
-        """
+
+
+        Changing the default editor hook:
+
+        If you wish to write your own editor hook, you can put it in a
+        configuration file which you load at startup time.  The default hook
+        is defined in the IPython.hooks module, and you can use that as a
+        starting example for further modifications.  That file also has
+        general instructions on how to set a new hook for use once you've
+        defined it."""
+        
         # FIXME: This function has become a convoluted mess.  It needs a
         # ground-up rewrite with clean, simple logic.
 
@@ -1603,10 +1697,13 @@ Currently the magic system has the following functions:\n"""
 
         opts,args = self.parse_options(parameter_s,'px')
 
+        # Default line number value
+        lineno = None
         if opts.has_key('p'):
             args = '_%s' % last_call[0]
             if not self.shell.user_ns.has_key(args):
                 args = last_call[1]
+            
         # use last_call to remember the state of the previous call, but don't
         # let it be clobbered by successive '-p' calls.
         try:
@@ -1619,8 +1716,6 @@ Currently the magic system has the following functions:\n"""
         # by default this is done with temp files, except when the given
         # arg is a filename
         use_temp = 1
-        # marker for at which line to open the file (for existing objects)
-        linemark = ''
 
         if re.match(r'\d',args):
             # Mode where user specifies ranges of lines, like in %macro.
@@ -1666,15 +1761,11 @@ Currently the magic system has the following functions:\n"""
                     try:
                         lineno = inspect.getsourcelines(data)[1]
                     except IOError:
-                        lineno = None
                         filename = make_filename(args)
                         if filename is None:
-                            warn('The file `%s` where `%s` was defined cannot be read.'
-                                 % (filename,data))
+                            warn('The file `%s` where `%s` was defined cannot '
+                                 'be read.' % (filename,data))
                             return
-                    # Decent Unix editors know how to start at a given line
-                    if os.name == 'posix' and lineno is not None:
-                        linemark = '+%s' % lineno
                 use_temp = 0
         else:
             data = ''
@@ -1691,16 +1782,21 @@ Currently the magic system has the following functions:\n"""
         # do actual editing here
         print 'Editing...',
         sys.stdout.flush()
-        xsys('%s %s %s' % (self.shell.rc.editor,linemark,filename))
+        self.shell.hooks.editor(filename,lineno)
         if opts.has_key('x'):  # -x prevents actual execution
             print
         else:
             print 'done. Executing edited code...'
             try:
                 execfile(filename,self.shell.user_ns)
-            except IOError:
-                warn('File not found. Did you forget to save?')
-                return
+            except IOError,msg:
+                if msg.filename == filename:
+                    warn('File not found. Did you forget to save?')
+                    return
+                else:
+                    self.shell.showtraceback()
+            except:
+                self.shell.showtraceback()
         if use_temp:
             contents = open(filename).read()
             return contents
@@ -2157,7 +2253,63 @@ Defaulting color scheme to 'NoColor'"""
           assigning it to the given variable.  By default the output is stored
           as a single string.
 
-          -v: verbose.  Print the contents of the variable."""
+          -v: verbose.  Print the contents of the variable.
+
+        In most cases you should not need to split as a list, because the
+        returned value is a special type of string which can automatically
+        provide its contents either as a list (split on newlines) or as a
+        space-separated string.  These are convenient, respectively, either
+        for sequential processing or to be passed to a shell command.
+
+        For example:
+
+            # Capture into variable a
+            In [9]: sc a=ls *py
+
+            # a is a string with embedded newlines
+            In [10]: a
+            Out[10]: 'setup.py\nwin32_manual_post_install.py'
+
+            # which can be seen as a list:
+            In [11]: a.l
+            Out[11]: ['setup.py', 'win32_manual_post_install.py']
+
+            # or as a whitespace-separated string:
+            In [12]: a.s
+            Out[12]: 'setup.py win32_manual_post_install.py'
+
+            # a.s is useful to pass as a single command line:
+            In [13]: !wc -l $a.s
+              146 setup.py
+              130 win32_manual_post_install.py
+              276 total
+
+            # while the list form is useful to loop over:
+            In [14]: for f in a.l:
+               ....:      !wc -l $f
+               ....:
+            146 setup.py
+            130 win32_manual_post_install.py
+
+        Similiarly, the lists returned by the -l option are also special, in
+        the sense that you can equally invoke the .s attribute on them to
+        automatically get a whitespace-separated string from their contents:
+
+            In [1]: sc -l b=ls *py
+
+            In [2]: b
+            Out[2]: ['setup.py', 'win32_manual_post_install.py']
+
+            In [3]: b.s
+            Out[3]: 'setup.py win32_manual_post_install.py'
+
+        In summary, both the lists and strings used for ouptut capture have
+        the following special attributes:
+
+            .l (or .list) : value as list.
+            .n (or .nlstr): value as newline-separated string.
+            .s (or .spstr): value as space-separated string.
+        """
 
         opts,args = self.parse_options(parameter_s,'lv')
         # Try to get a variable name and command to run
@@ -2180,7 +2332,9 @@ Defaulting color scheme to 'NoColor'"""
         if err:
             print >> Term.cerr,err
         if opts.has_key('l'):
-            out = out.split('\n')
+            out = SList(out.split('\n'))
+        else:
+            out = LSString(out)
         if opts.has_key('v'):
             print '%s ==\n%s' % (var,pformat(out))
         self.shell.user_ns.update({var:out})
@@ -2209,14 +2363,66 @@ Defaulting color scheme to 'NoColor'"""
         like '%sc -l'.  The reason for this is to make it as easy as possible
         to process line-oriented shell output via further python commands.
         %sc is meant to provide much finer control, but requires more
-        typing."""
+        typing.
+
+        3) Just like %sc -l, this is a list with special attributes:
+
+          .l (or .list) : value as list.
+          .n (or .nlstr): value as newline-separated string.
+          .s (or .spstr): value as whitespace-separated string.
+
+        This is very useful when trying to use such lists as arguments to
+        system commands."""
 
         if parameter_s:
             out,err = self.shell.getoutputerror(parameter_s)
             if err:
                 print >> Term.cerr,err
-            return out.split('\n')
+            return SList(out.split('\n'))
 
+    def magic_bg(self, parameter_s=''):
+        """Run a job in the background, in a separate thread.
+
+        For example,
+
+          %bg myfunc(x,y,z=1)
+
+        will execute 'myfunc(x,y,z=1)' in a background thread.  As soon as the
+        execution starts, a message will be printed indicating the job
+        number.  If your job number is 5, you can use
+
+          myvar = jobs.result(5)  or  myvar = jobs[5].result
+
+        to assign this result to variable 'myvar'.
+
+        IPython has a job manager, accessible via the 'jobs' object.  You can
+        type jobs? to get more information about it, and use jobs.<TAB> to see
+        its attributes.  All attributes not starting with an underscore are
+        meant for public use.
+
+        In particular, look at the jobs.new() method, which is used to create
+        new jobs.  This magic %bg function is just a convenience wrapper
+        around jobs.new(), for expression-based jobs.  If you want to create a
+        new job with an explicit function object and arguments, you must call
+        jobs.new() directly.
+
+        The jobs.new docstring also describes in detail several important
+        caveats associated with a thread-based model for background job
+        execution.  Type jobs.new? for details.
+
+        You can check the status of all jobs with jobs.status().
+
+        The jobs variable is set by IPython into the Python builtin namespace.
+        If you ever declare a variable named 'jobs', you will shadow this
+        name.  You can either delete your global jobs variable to regain
+        access to the job manager, or make a new name and assign it manually
+        to the manager (stored in IPython's namespace).  For example, to
+        assign the job manager to the Jobs name, use:
+
+          Jobs = __builtins__.jobs"""
+        
+        self.shell.jobs.new(parameter_s,self.shell.user_ns)
+        
     def magic_bookmark(self, parameter_s=''):
         """Manage IPython's bookmark system.
 
