@@ -2,7 +2,7 @@
 """
 Classes for handling input/output prompts.
 
-$Id: Prompts.py,v 1.19 2004/11/30 08:53:59 fperez Exp $"""
+$Id: Prompts.py,v 1.25 2005/03/26 22:43:58 fperez Exp $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2004 Fernando Perez <fperez@colorado.edu>
@@ -27,7 +27,7 @@ from pprint import pprint,pformat
 from IPython.genutils import *
 from IPython.Struct import Struct
 from IPython.Magic import Macro
-from IPython.Itpl import Itpl
+from IPython.Itpl import ItplNS
 from IPython import ColorANSI
 
 #****************************************************************************
@@ -97,36 +97,19 @@ def multiple_replace(dict, text):
 # reasonable directory name will do, we just want the $HOME -> '~' operation
 # to become a no-op.  We pre-compute $HOME here so it's not done on every
 # prompt call.
+
+# FIXME:
+
+# - This should be turned into a class which does proper namespace management,
+# since the prompt specials need to be evaluated in a certain namespace.
+# Currently it's just globals, which need to be managed manually by code
+# below.
+
+# - I also need to split up the color schemes from the prompt specials
+# somehow.  I don't have a clean design for that quite yet.
+
 HOME = os.environ.get("HOME","//////:::::ZZZZZ,,,~~~")
 
-def cwd_filt(depth):
-    """Return the last depth elements of the current working directory.
-
-    $HOME is always replaced with '~'.
-    If depth==0, the full path is returned."""
-    
-    cwd = os.getcwd().replace(HOME,"~")
-    out = os.sep.join(cwd.split(os.sep)[-depth:])
-    if out:
-        return out
-    else:
-        return os.sep
-
-def cwd_filt2(depth):
-    """Return the last depth elements of the current working directory.
-
-    $HOME is always replaced with '~'.
-    If depth==0, the full path is returned."""
-    
-    cwd = os.getcwd().replace(HOME,"~").split(os.sep)
-    if '~' in cwd and len(cwd) == depth+1:
-        depth += 1
-    out = os.sep.join(cwd[-depth:])
-    if out:
-        return out
-    else:
-        return os.sep
-
 # We precompute a few more strings here for the prompt_specials, which are
 # fixed once ipython starts.  This reduces the runtime overhead of computing
 # prompt strings.
@@ -152,19 +135,19 @@ prompt_specials_color = {
     # These X<N> are an extension to the normal bash prompts.  They return
     # N terms of the path, after replacing $HOME with '~'
     '\\X0': '${os.getcwd().replace("%s","~")}' % HOME,
-    '\\X1': '${cwd_filt(1)}',
-    '\\X2': '${cwd_filt(2)}',
-    '\\X3': '${cwd_filt(3)}',
-    '\\X4': '${cwd_filt(4)}',
-    '\\X5': '${cwd_filt(5)}',
+    '\\X1': '${self.cwd_filt(1)}',
+    '\\X2': '${self.cwd_filt(2)}',
+    '\\X3': '${self.cwd_filt(3)}',
+    '\\X4': '${self.cwd_filt(4)}',
+    '\\X5': '${self.cwd_filt(5)}',
     # Y<N> are similar to X<N>, but they show '~' if it's the directory
     # N+1 in the list.  Somewhat like %cN in tcsh.
-    '\\Y0': '${cwd_filt2(0)}',
-    '\\Y1': '${cwd_filt2(1)}',
-    '\\Y2': '${cwd_filt2(2)}',
-    '\\Y3': '${cwd_filt2(3)}',
-    '\\Y4': '${cwd_filt2(4)}',
-    '\\Y5': '${cwd_filt2(5)}',
+    '\\Y0': '${self.cwd_filt2(0)}',
+    '\\Y1': '${self.cwd_filt2(1)}',
+    '\\Y2': '${self.cwd_filt2(2)}',
+    '\\Y3': '${self.cwd_filt2(3)}',
+    '\\Y4': '${self.cwd_filt2(4)}',
+    '\\Y5': '${self.cwd_filt2(5)}',
     # Hostname up to first .
     '\\h': HOSTNAME_SHORT,
     # Full hostname
@@ -194,19 +177,32 @@ prompt_specials_nocolor['\\#'] = '${self.cache.prompt_count}'
 # with a color name which may begin with a letter used by any other of the
 # allowed specials.  This of course means that \\C will never be allowed for
 # anything else.
-for _color in dir(ColorANSI.InputTermColors):
+input_colors = ColorANSI.InputTermColors
+for _color in dir(input_colors):
     if _color[0] != '_':
-        prompt_specials_color['\\C_%s' % _color] = \
-                                       getattr(ColorANSI.InputTermColors,_color)
-        prompt_specials_nocolor['\\C_%s' % _color] = ''
+        c_name = '\\C_'+_color
+        prompt_specials_color[c_name] = getattr(input_colors,_color)
+        prompt_specials_nocolor[c_name] = ''
 
 # we default to no color for safety.  Note that prompt_specials is a global
 # variable used by all prompt objects.
 prompt_specials = prompt_specials_nocolor
+
 #-----------------------------------------------------------------------------
+def str_safe(arg):
+    """Convert to a string, without ever raising an exception.
+
+    If str(arg) fails, <ERROR: ... > is returned, where ... is the exception
+    error message."""
+    
+    try:
+        return str(arg)
+    except Exception,msg:
+        return '<ERROR: %s>' % msg
+
 class BasePrompt:
     """Interactive prompt similar to Mathematica's."""
-    def __init__(self,cache,sep,prompt):
+    def __init__(self,cache,sep,prompt,pad_left=False):
 
         # Hack: we access information about the primary prompt through the
         # cache argument.  We need this, because we want the secondary prompt
@@ -214,6 +210,13 @@ class BasePrompt:
         # by all prompt classes through the cache.  Nice OO spaghetti code!
         self.cache = cache
         self.sep = sep
+        
+        # regexp to count the number of spaces at the end of a prompt
+        # expression, useful for prompt auto-rewriting
+        self.rspace = re.compile(r'(\s*)$')
+        # Flag to left-pad prompt strings to match the length of the primary
+        # prompt
+        self.pad_left = pad_left
         # Set template to create each actual prompt (where numbers change)
         self.p_template = prompt
         self.set_p_str()
@@ -223,32 +226,75 @@ class BasePrompt:
 
         This must be called every time the color settings change, because the
         prompt_specials global may have changed."""
+
+        import os,time  # needed in locals for prompt string handling
+        loc = locals()
+        self.p_str = ItplNS('%s%s%s' %
+                            ('${self.sep}${self.col_p}',
+                             multiple_replace(prompt_specials, self.p_template),
+                             '${self.col_norm}'),self.cache.user_ns,loc)
         
-        self.p_str = Itpl('%s%s%s ' %
-                          ('$self.sep${self.col_p}',
-                           multiple_replace(prompt_specials, self.p_template),
-                          '$self.col_norm'))
-        self.p_str_nocolor = Itpl(multiple_replace(prompt_specials_nocolor,
-                                                   self.p_template))
+        self.p_str_nocolor = ItplNS(multiple_replace(prompt_specials_nocolor,
+                                                     self.p_template),
+                                    self.cache.user_ns,loc)
+
+    def write(self,msg):  # dbg
+        sys.stdout.write(msg)
+        return ''
 
     def __str__(self):
         """Return a string form of the prompt.
 
-        This for is useful for continuation and output prompts, since it
-        matches lengths with the primary one."""
+        This for is useful for continuation and output prompts, since it is
+        left-padded to match lengths with the primary one (if the
+        self.pad_left attribute is set)."""
 
-        # We must find the amount of padding required to match lengths, taking
-        # the color escapes (which are invisible on-screen) into account.
-        out_str = str(self.p_str)
-        esc_pad = len(out_str) - len(str(self.p_str_nocolor))
-        format = '%%%ss' % (len(str(self.cache.last_prompt))+esc_pad)
-        return format % out_str
+        out_str = str_safe(self.p_str)
+        if self.pad_left:
+            # We must find the amount of padding required to match lengths,
+            # taking the color escapes (which are invisible on-screen) into
+            # account.
+            esc_pad = len(out_str) - len(str_safe(self.p_str_nocolor))
+            format = '%%%ss' % (len(str(self.cache.last_prompt))+esc_pad)
+            return format % out_str
+        else:
+            return out_str
+
+    # these path filters are put in as methods so that we can control the
+    # namespace where the prompt strings get evaluated
+    def cwd_filt(self,depth):
+        """Return the last depth elements of the current working directory.
+
+        $HOME is always replaced with '~'.
+        If depth==0, the full path is returned."""
+
+        cwd = os.getcwd().replace(HOME,"~")
+        out = os.sep.join(cwd.split(os.sep)[-depth:])
+        if out:
+            return out
+        else:
+            return os.sep
+
+    def cwd_filt2(self,depth):
+        """Return the last depth elements of the current working directory.
+
+        $HOME is always replaced with '~'.
+        If depth==0, the full path is returned."""
+
+        cwd = os.getcwd().replace(HOME,"~").split(os.sep)
+        if '~' in cwd and len(cwd) == depth+1:
+            depth += 1
+        out = os.sep.join(cwd[-depth:])
+        if out:
+            return out
+        else:
+            return os.sep
 
 class Prompt1(BasePrompt):
     """Input interactive prompt similar to Mathematica's."""
 
-    def __init__(self,cache,sep='\n',prompt='In [\\#]:'):
-        BasePrompt.__init__(self,cache,sep,prompt)
+    def __init__(self,cache,sep='\n',prompt='In [\\#]: ',pad_left=True):
+        BasePrompt.__init__(self,cache,sep,prompt,pad_left)
 
     def set_colors(self):
         self.set_p_str()
@@ -263,8 +309,8 @@ class Prompt1(BasePrompt):
         
     def __str__(self):
         self.cache.prompt_count += 1
-        self.cache.last_prompt = str(self.p_str_nocolor)
-        return str(self.p_str)
+        self.cache.last_prompt = str_safe(self.p_str_nocolor).split('\n')[-1]
+        return str_safe(self.p_str)
 
     def auto_rewrite(self):
         """Print a string of the form '--->' which lines up with the previous
@@ -272,13 +318,15 @@ class Prompt1(BasePrompt):
         handling automatically special syntaxes."""
 
         curr = str(self.cache.last_prompt)
-        return self.col_p_ni + '-'*(len(curr)-1)+'> ' + self.col_norm_ni
+        nrspaces = len(self.rspace.search(curr).group())
+        return '%s%s>%s%s' % (self.col_p_ni,'-'*(len(curr)-nrspaces-1),
+                              ' '*nrspaces,self.col_norm_ni)
 
 class PromptOut(BasePrompt):
     """Output interactive prompt similar to Mathematica's."""
 
-    def __init__(self,cache,sep='',prompt='Out[\\#]:'):
-        BasePrompt.__init__(self,cache,sep,prompt)
+    def __init__(self,cache,sep='',prompt='Out[\\#]: ',pad_left=True):
+        BasePrompt.__init__(self,cache,sep,prompt,pad_left)
         if not self.p_template:
             self.__str__ = lambda: ''
 
@@ -292,18 +340,23 @@ class PromptOut(BasePrompt):
 class Prompt2(BasePrompt):
     """Interactive continuation prompt."""
     
-    def __init__(self,cache,prompt='   .\\D.:'):
+    def __init__(self,cache,prompt='   .\\D.: ',pad_left=True):
         self.cache = cache
         self.p_template = prompt
+        self.pad_left = pad_left
         self.set_p_str()
 
     def set_p_str(self):
-        self.p_str = Itpl('%s%s%s ' %
-                          ('${self.col_p2}',
-                           multiple_replace(prompt_specials, self.p_template),
-                          '$self.col_norm'))
-        self.p_str_nocolor = Itpl(multiple_replace(prompt_specials_nocolor,
-                                                   self.p_template))
+        import os,time  # needed in locals for prompt string handling
+        loc = locals()
+        self.p_str = ItplNS('%s%s%s' %
+                            ('${self.col_p2}',
+                             multiple_replace(prompt_specials, self.p_template),
+                             '$self.col_norm'),
+                            self.cache.user_ns,loc)
+        self.p_str_nocolor = ItplNS(multiple_replace(prompt_specials_nocolor,
+                                                     self.p_template),
+                                    self.cache.user_ns,loc)
 
     def set_colors(self):
         self.set_p_str()
@@ -330,7 +383,7 @@ class CachedOutput:
     def __init__(self,cache_size,Pprint,colors='NoColor',input_sep='\n',
                  output_sep='\n',output_sep2='',user_ns={},
                  ps1 = None, ps2 = None,ps_out = None,
-                 input_hist = None):
+                 input_hist = None,pad_left=True):
 
         cache_size_min = 20
         if cache_size <= 0:
@@ -347,17 +400,24 @@ class CachedOutput:
         self.cache_size = cache_size
         self.input_sep = input_sep
 
+        # we need a reference to the user-level namespace
+        self.user_ns = user_ns
+        # and to the user's input
+        self.input_hist = input_hist
+
         # Set input prompt strings and colors
         if cache_size == 0:
-            if ps1.find('%n') > -1 or ps1.find('\\#') > -1: ps1 = '>>>'
-            if ps2.find('%n') > -1 or ps2.find('\\#') > -1: ps2 = '...'
-        self.ps1_str = self._set_prompt_str(ps1,'In [\\#]:','>>>')
-        self.ps2_str = self._set_prompt_str(ps2,'   .\\D.:','...')
-        self.ps_out_str = self._set_prompt_str(ps_out,'Out[\\#]:','')
+            if ps1.find('%n') > -1 or ps1.find('\\#') > -1: ps1 = '>>> '
+            if ps2.find('%n') > -1 or ps2.find('\\#') > -1: ps2 = '... '
+        self.ps1_str = self._set_prompt_str(ps1,'In [\\#]: ','>>> ')
+        self.ps2_str = self._set_prompt_str(ps2,'   .\\D.: ','... ')
+        self.ps_out_str = self._set_prompt_str(ps_out,'Out[\\#]: ','')
 
-        self.prompt1 = Prompt1(self,sep=input_sep,prompt=self.ps1_str)
-        self.prompt2 = Prompt2(self,prompt=self.ps2_str)
-        self.prompt_out = PromptOut(self,sep='',prompt=self.ps_out_str)
+        self.prompt1 = Prompt1(self,sep=input_sep,prompt=self.ps1_str,
+                               pad_left=pad_left)
+        self.prompt2 = Prompt2(self,prompt=self.ps2_str,pad_left=pad_left)
+        self.prompt_out = PromptOut(self,sep='',prompt=self.ps_out_str,
+                                    pad_left=pad_left)
         self.color_table = PromptColors
         self.set_colors(colors)
 
@@ -375,22 +435,18 @@ class CachedOutput:
         self._,self.__,self.___ = '','',''
         self.pprint_types = map(type,[(),[],{}])
         
-        # we need a reference to the user-level namespace
-        self.user_ns = user_ns
-        # and to the user's input
-        self.input_hist = input_hist
         # these are deliberately global:
         to_user_ns = {'_':self._,'__':self.__,'___':self.___}
         self.user_ns.update(to_user_ns)
 
     def _set_prompt_str(self,p_str,cache_def,no_cache_def):
-        if p_str:
-            return p_str
-        else:
+        if p_str is None:
             if self.do_full_cache:
                 return cache_def
             else:
                 return no_cache_def
+        else:
+            return p_str
                 
     def set_colors(self,colors):
         """Set the active color scheme and configure colors for the three
@@ -412,8 +468,8 @@ class CachedOutput:
     def __call__(self,arg=None):
         """Printing with history cache management.
         
-        This is invoked everytime the interpreter needs to print, and is activated
-        by setting the variable sys.displayhook to it."""
+        This is invoked everytime the interpreter needs to print, and is
+        activated by setting the variable sys.displayhook to it."""
 
         # If something injected a '_' variable in __builtin__, delete
         # ipython's automatic one so we don't clobber that.  gettext() in
@@ -429,14 +485,15 @@ class CachedOutput:
             # do not print output if input ends in ';'
             if self.input_hist[self.prompt_count].endswith(';\n'):
                 return
-
-            Term.cout.write(self.output_sep)  # don't use print, puts an extra space
+            # don't use print, puts an extra space
+            Term.cout.write(self.output_sep)
             if self.do_full_cache:
                 Term.cout.write(str(self.prompt_out))
 
             if isinstance(arg,Macro):
                 print 'Executing Macro...'
-                Term.cout.flush()  # in case the macro takes a long time to execute
+                # in case the macro takes a long time to execute
+                Term.cout.flush()
                 exec arg.value in self.user_ns
                 return None
 
@@ -487,7 +544,6 @@ class CachedOutput:
             self.__ = self._
             self._ = arg
             self.user_ns.update({'_':self._,'__':self.__,'___':self.___})
-            #to_main.update({'_':self._,'__':self.__,'___':self.___})
             
         # hackish access to top-level  namespace to create _1,_2... dynamically
         to_main = {}
