@@ -4,7 +4,7 @@
 All the matplotlib support code was co-developed with John Hunter,
 matplotlib's author.
 
-$Id: Shell.py,v 1.39 2005/01/23 05:58:39 fperez Exp $"""
+$Id: Shell.py,v 1.44 2005/04/02 07:28:38 fperez Exp $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2004 Fernando Perez <fperez@colorado.edu>
@@ -48,7 +48,7 @@ USE_TK = False
 class IPShell:
     """Create an IPython instance."""
     
-    def __init__(self,argv=None,user_ns=None,debug=0,
+    def __init__(self,argv=None,user_ns=None,debug=1,
                  shell_class=InteractiveShell):
         self.IP = make_IPython(argv,user_ns=user_ns,debug=debug,
                                shell_class=shell_class)
@@ -133,13 +133,15 @@ class IPShellEmbed:
         self.IP = make_IPython(argv,rc_override=rc_override,embedded=True)
 
         self.IP.name_space_init()
-        # mark this as an embedded instance so we know if we get a crash post-mortem
+        # mark this as an embedded instance so we know if we get a crash
+        # post-mortem
         self.IP.rc.embedded = 1
         # copy our own displayhook also
         self.sys_displayhook_embed = sys.displayhook
         # and leave the system's display hook clean
         sys.displayhook = self.sys_displayhook_ori
-        # don't use the ipython crash handler so that user exceptions aren't trapped
+        # don't use the ipython crash handler so that user exceptions aren't
+        # trapped
         sys.excepthook = ultraTB.FormattedTB(color_scheme = self.IP.rc.colors,
                                              mode = self.IP.rc.xmode,
                                              call_pdb = self.IP.rc.pdb)
@@ -269,12 +271,8 @@ class MTInteractiveShell(InteractiveShell):
         
         IPython.iplib.InteractiveShell.__init__(self,name,usage,rc,user_ns,banner2)
 
-        # Object variable to store code object waiting execution.  No need to
-        # use a Queue here, since it's a single item which gets cleared once run.
-        self.code_to_run = None
-
         # Locking control variable
-        self.ready = threading.Condition()
+        self.thread_ready = threading.Condition()
 
         # Stuff to do at closing time
         self._kill = False
@@ -313,10 +311,11 @@ class MTInteractiveShell(InteractiveShell):
 
         # Case 3
         # Store code in self, so the execution thread can handle it
-        self.ready.acquire()
+        self.thread_ready.acquire()
+        self.code_to_run_src = source
         self.code_to_run = code
-        self.ready.wait()  # Wait until processed in timeout interval
-        self.ready.release()
+        self.thread_ready.wait()  # Wait until processed in timeout interval
+        self.thread_ready.release()
 
         return False
 
@@ -326,7 +325,7 @@ class MTInteractiveShell(InteractiveShell):
         Multithreaded wrapper around IPython's runcode()."""
 
         # lock thread-protected stuff
-        self.ready.acquire()
+        self.thread_ready.acquire()
 
         # Install sigint handler
         try:
@@ -345,21 +344,19 @@ class MTInteractiveShell(InteractiveShell):
 
         # Run pending code by calling parent class
         if self.code_to_run is not None:
-            self.ready.notify()
+            self.thread_ready.notify()
             InteractiveShell.runcode(self,self.code_to_run)
-            # Flush out code object which has been run
-            self.code_to_run = None
             
         # We're done with thread-protected variables
-        self.ready.release()
+        self.thread_ready.release()
         # This MUST return true for gtk threading to work
         return True
 
     def kill (self):
         """Kill the thread, returning when it has been shut down."""
-        self.ready.acquire()
+        self.thread_ready.acquire()
         self._kill = True
-        self.ready.release()
+        self.thread_ready.release()
 
 class MatplotlibShellBase:
     """Mixin class to provide the necessary modifications to regular IPython
@@ -414,11 +411,6 @@ class MatplotlibShellBase:
         matplotlib.backends.show._needmain = False
         self.mpl_backend = matplotlib.rcParams['backend']
 
-##        # If the user forgets to call show(), we do it for them but give a
-##        # warning.  Since we don't want to print this warning every time, we
-##        # need a flag to track its use
-##        self.mpl_autoshow_warned = False
-
         # we also need to block switching of interactive backends by use()
         self.mpl_use = matplotlib.use
         self.mpl_use._called = False
@@ -427,7 +419,6 @@ class MatplotlibShellBase:
 
         # We need to detect at runtime whether show() is called by the user.
         # For this, we wrap it into a decorator which adds a 'called' flag.
-##        backend.show = flag_calls(backend.show)
         backend.draw_if_interactive = flag_calls(backend.draw_if_interactive)
 
         # This must be imported last in the matplotlib series, after
@@ -455,10 +446,9 @@ class MatplotlibShellBase:
         
         # Build matplotlib info banner
         b="""
-  Welcome to pylab, a matplotlib-based Python environment
-    help(matplotlib) -> generic matplotlib information
-    help(%s)      -> matlab-compatible commands from matplotlib
-    help(plotting)   -> plotting commands\n""" % pname
+  Welcome to pylab, a matplotlib-based Python environment.
+  For more information, type 'help(pylab)'.
+  """
 
         return user_ns,b
 
@@ -478,22 +468,6 @@ class MatplotlibShellBase:
         if self.pylab.draw_if_interactive.called:
             self.pylab.draw()
             self.pylab.draw_if_interactive.called = False
-
-##            # Let's be nice and call show() for the user if he didn't, but
-##            # give a warning (once).  I won't make this warning optional,
-##            # because not calling show() is potentially a bug: such a script
-##            # will produce no visible output outside of ipython, so it's
-##            # better to force users to write valid matplotlib code.
-##            if not self.pylab.show.called:
-##                self.pylab.show()
-##                self.pylab.show.called = False
-##                if not self.mpl_autoshow_warned:
-##                    print ("Did you forget to call matplotlib's show()?\n"
-##                           "IPython has called show() for you, but if "
-##                           "you run this script \n"
-##                           "outside of IPython, it will not display any plots.\n"
-##                           "This warning appears only once per session.")
-##                    self.mpl_autoshow_warned = True
                 
         # if a backend switch was performed, reverse it now
         if self.mpl_use._called:
@@ -624,7 +598,7 @@ class IPShellGTK(threading.Thread):
     
     TIMEOUT = 100 # Millisecond interval between timeouts.
 
-    def __init__(self,argv=None,user_ns=None,debug=0,
+    def __init__(self,argv=None,user_ns=None,debug=1,
                  shell_class=MTInteractiveShell):
 
         import pygtk
@@ -669,7 +643,9 @@ class IPShellGTK(threading.Thread):
                       'current state!')
                 self.IP.InteractiveTB()
         self.start()
+        self.gtk.threads_enter()
         self.gtk_mainloop()
+        self.gtk.threads_leave()
         self.join()
 
     def on_timer(self):
@@ -686,7 +662,7 @@ class IPShellWX(threading.Thread):
     
     TIMEOUT = 100 # Millisecond interval between timeouts.
 
-    def __init__(self,argv=None,user_ns=None,debug=0,
+    def __init__(self,argv=None,user_ns=None,debug=1,
                  shell_class=MTInteractiveShell):
 
         import wxPython.wx as wx
@@ -760,7 +736,7 @@ class IPShellMatplotlib(IPShell):
 
     Having this on a separate class simplifies the external driver code."""
     
-    def __init__(self,argv=None,user_ns=None,debug=0):
+    def __init__(self,argv=None,user_ns=None,debug=1):
         IPShell.__init__(self,argv,user_ns,debug,shell_class=MatplotlibShell)
 
 class IPShellMatplotlibGTK(IPShellGTK):
@@ -768,7 +744,7 @@ class IPShellMatplotlibGTK(IPShellGTK):
 
     Multi-threaded class, meant for the GTK* backends."""
     
-    def __init__(self,argv=None,user_ns=None,debug=0):
+    def __init__(self,argv=None,user_ns=None,debug=1):
         IPShellGTK.__init__(self,argv,user_ns,debug,shell_class=MatplotlibMTShell)
 
 class IPShellMatplotlibWX(IPShellWX):
@@ -776,7 +752,7 @@ class IPShellMatplotlibWX(IPShellWX):
 
     Multi-threaded class, meant for the WX* backends."""
     
-    def __init__(self,argv=None,user_ns=None,debug=0):
+    def __init__(self,argv=None,user_ns=None,debug=1):
         IPShellWX.__init__(self,argv,user_ns,debug,shell_class=MatplotlibMTShell)
 
 #-----------------------------------------------------------------------------
