@@ -27,7 +27,7 @@ IPython tries to:
 
 IPython requires Python 2.2 or newer.
 
-$Id: __init__.py,v 1.5 2004/11/04 07:58:16 fperez Exp $"""
+$Id: __init__.py,v 1.6 2005/03/02 07:11:15 fperez Exp $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2004 Fernando Perez. <fperez@colorado.edu>
@@ -42,7 +42,7 @@ if sys.version[0:3] < '2.2':
     raise ImportError, 'Python Version 2.2 or above is required.'
         
 # Define what gets imported with a 'from IPython import *'
-__all__ = ['deep_reload','genutils','ultraTB','DPyGetOpt','Itpl',
+__all__ = ['deep_reload','genutils','ultraTB','DPyGetOpt','Itpl','hooks',
            'ConfigLoader','OutputTrap','Release','Struct','Shell']
 
 # Load __all__ in IPython namespace so that a simple 'import IPython' gives
