@@ -6,7 +6,7 @@ Requires Python 2.1 or better.
 
 This file contains the main make_IPython() starter function.
 
-$Id: ipmaker.py,v 1.26 2004/12/13 20:36:58 fperez Exp $"""
+$Id: ipmaker.py,v 1.32 2005/04/07 00:09:05 fperez Exp $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2004 Fernando Perez. <fperez@colorado.edu>
@@ -51,7 +51,7 @@ from IPython.Prompts import CachedOutput
 from IPython.genutils import *
 
 #-----------------------------------------------------------------------------
-def make_IPython(argv=None,user_ns=None,debug=0,rc_override=None,
+def make_IPython(argv=None,user_ns=None,debug=1,rc_override=None,
                  shell_class=InteractiveShell,embedded=False,**kw):
     """This is a dump of IPython into a single function.
 
@@ -67,6 +67,9 @@ def make_IPython(argv=None,user_ns=None,debug=0,rc_override=None,
 
     #----------------------------------------------------------------------
     # Defaults and initialization
+    
+    # For developer debugging, deactivates crash handler and uses pdb.
+    DEVDEBUG = False
 
     if argv is None:
         argv = sys.argv
@@ -96,13 +99,8 @@ def make_IPython(argv=None,user_ns=None,debug=0,rc_override=None,
     else:
         IP.user_ns['help'] = _Helper()
 
-    if debug:
-        # For developer debugging only.  Activate this by changing the main
-        # ipython script from:
-        #   IPython.Shell.IPShell().mainloop()
-        # to:
-        #   IPython.Shell.IPShell(debug=1).mainloop()
-        #
+    if DEVDEBUG:
+        # For developer debugging only (global flag)
         from IPython import ultraTB
         sys.excepthook = ultraTB.VerboseTB(call_pdb=1)
     else:
@@ -125,15 +123,17 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
 
     IP.usage = interactive_usage
 
-    # default directory for configuration
-    ipythondir = os.path.abspath(os.environ.get('IPYTHONDIR',
-                                 os.path.join(IP.home_dir,'.ipython') ) )
-
-    # suffix to use in all rc-files:
+    # Platform-dependent suffix and directory names
     if os.name == 'posix':
         rc_suffix = ''
+        ipdir_def = '.ipython'
     else:
         rc_suffix = '.ini'
+        ipdir_def = '_ipython'
+
+    # default directory for configuration
+    ipythondir = os.path.abspath(os.environ.get('IPYTHONDIR',
+                                 os.path.join(IP.home_dir,ipdir_def)))
 
     # we need the directory where IPython itself is installed
     import IPython
@@ -159,9 +159,10 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                     'c=s classic|cl color_info! colors=s confirm_exit! '
                     'debug! deep_reload! editor=s log|l messages! nosep pdb! '
                     'pprint! prompt_in1|pi1=s prompt_in2|pi2=s prompt_out|po=s '
-                    'quick screen_length|sl=i '
+                    'quick screen_length|sl=i prompts_pad_left=i '
                     'logfile|lf=s logplay|lp=s profile|p=s '
-                    'readline! readline_omit__names! '
+                    'readline! readline_merge_completions! '
+                    'readline_omit__names! '
                     'rcfile=s separate_in|si=s separate_out|so=s '
                     'separate_out2|so2=s xmode=s '
                     'magic_docstrings system_verbose! '
@@ -198,7 +199,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                       log = 0,
                       logfile = '',
                       logplay = '',
-                      multi_line_specials = 0,
+                      multi_line_specials = 1,
                       messages = 1,
                       nosep = 0,
                       pdb = 0,
@@ -207,8 +208,10 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                       prompt_in1 = 'In [\\#]:',
                       prompt_in2 = '   .\\D.:',
                       prompt_out = 'Out[\\#]:',
+                      prompts_pad_left = 1,
                       quick = 0,
                       readline = 1,
+                      readline_merge_completions = 1,
                       readline_omit__names = 0,
                       rcfile = 'ipythonrc' + rc_suffix,
                       screen_length = 0,
@@ -329,8 +332,8 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
     # Log replay
     
     # if -logplay, we need to 'become' the other session. That basically means
-    # replacing the current command line environment with that of the old session
-    # and moving on.
+    # replacing the current command line environment with that of the old
+    # session and moving on.
 
     # this is needed so that later we know we're in session reload mode, as
     # opts_all will get overwritten:
@@ -356,8 +359,8 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                 # this reloads that session's command line
                 cmd = logplay.readline()[6:] 
                 exec cmd
-                # restore the true debug flag given so that the process of session
-                # loading itself can be monitored.
+                # restore the true debug flag given so that the process of
+                # session loading itself can be monitored.
                 opts.debug = opts_debug_save
                 # save the logplay flag so later we don't overwrite the log
                 opts.logplay = load_logplay
@@ -484,8 +487,8 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
         IP.rc.quick = 1
         IP.rc.cache_size = 0
         IP.rc.pprint = 0
-        IP.rc.prompt_in1 = '>>>'
-        IP.rc.prompt_in2 = '...'
+        IP.rc.prompt_in1 = '>>> '
+        IP.rc.prompt_in2 = '... '
         IP.rc.prompt_out = ''
         IP.rc.separate_in = IP.rc.separate_out = IP.rc.separate_out2 = '0'
         IP.rc.colors = 'NoColor'
@@ -688,7 +691,8 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                                   ps2 = IP.rc.prompt_in2,
                                   ps_out = IP.rc.prompt_out,
                                   user_ns = IP.user_ns,
-                                  input_hist = IP.input_hist)
+                                  input_hist = IP.input_hist,
+                                  pad_left = IP.rc.prompts_pad_left)
 
     # Set user colors (don't do it in the constructor above so that it doesn't
     # crash if colors option is invalid)
